@@ -10,6 +10,7 @@ typedef struct _E_Config_Binding_Edge       E_Config_Binding_Edge;
 typedef struct _E_Config_Binding_Signal     E_Config_Binding_Signal;
 typedef struct _E_Config_Binding_Wheel      E_Config_Binding_Wheel;
 typedef struct _E_Config_Binding_Acpi       E_Config_Binding_Acpi;
+typedef struct _E_Config_Binding_Swipe      E_Config_Binding_Swipe;
 typedef struct _E_Config_Desktop_Background E_Config_Desktop_Background;
 typedef struct _E_Config_Desklock_Background E_Config_Desklock_Background;
 typedef struct _E_Config_Desktop_Name       E_Config_Desktop_Name;
@@ -46,7 +47,7 @@ typedef enum
 /* increment this whenever a new set of config values are added but the users
  * config doesn't need to be wiped - simply new values need to be put in
  */
-#define E_CONFIG_FILE_GENERATION 27
+#define E_CONFIG_FILE_GENERATION 35
 #define E_CONFIG_FILE_VERSION    ((E_CONFIG_FILE_EPOCH * 1000000) + E_CONFIG_FILE_GENERATION)
 
 #define E_CONFIG_BINDINGS_VERSION 0 // DO NOT INCREMENT UNLESS YOU WANT TO WIPE ALL BINDINGS!!!!!
@@ -55,7 +56,6 @@ struct _E_Config
 {
    int         config_version; // INTERNAL
    E_Config_Profile_Type config_type; // INTERNAL
-   int         show_splash; // GUI
    const char *desktop_default_background; // GUI
    Eina_List  *desktop_backgrounds; // GUI
    const char *desktop_default_name; // DEAD
@@ -69,13 +69,11 @@ struct _E_Config
    int         border_shade_animate; // GUI
    int         border_shade_transition; // GUI
    double      border_shade_speed; // GUI
-   double      framerate; // GUI
    int         priority; // GUI
    int         zone_desks_x_count; // GUI
    int         zone_desks_y_count; // GUI
    int         show_desktop_icons; // GUI
    int         edge_flip_dragging; // GUI
-   int         no_module_delay; // GUI
    const char *language; // GUI
    const char *desklock_language; // GUI
    Eina_List  *modules; // GUI
@@ -90,6 +88,7 @@ struct _E_Config
    Eina_List  *signal_bindings; // GUI
    Eina_List  *wheel_bindings; // GUI
    Eina_List  *acpi_bindings; // GUI
+   Eina_List  *swipe_bindings; // GUI
 
    Eina_List  *path_append_data; // GUI
    Eina_List  *path_append_images; // GUI
@@ -132,14 +131,10 @@ struct _E_Config
    int         winlist_list_focus_while_selecting; // GUI
    int         winlist_list_raise_while_selecting; // GUI
    int         winlist_list_move_after_select; // GUI
-   double      winlist_pos_align_x; // GUI
-   double      winlist_pos_align_y; // GUI
-   double      winlist_pos_size_w; // GUI
-   double      winlist_pos_size_h; // GUI
-   int         winlist_pos_min_w; // GUI
-   int         winlist_pos_min_h; // GUI
-   int         winlist_pos_max_w; // GUI
-   int         winlist_pos_max_h; // GUI
+   int         winlist_mode; // GUI
+   double      winlist_list_size; // GUI
+   double      winlist_large_size; // GUI
+   Eina_Bool   winlist_list_no_miniatures; // GUI
    int         maximize_policy; // GUI
    int         allow_manip; // GUI
    int         border_fix_on_shelf_toggle; // GUI
@@ -170,8 +165,7 @@ struct _E_Config
    int         menu_autoscroll_margin; // GUI
    int         menu_autoscroll_cursor_margin; // GUI
    const char *input_method; // GUI
-   struct
-   {
+   struct {
       int move;      // GUI
       int resize;      // GUI
       int raise;      // GUI
@@ -185,7 +179,7 @@ struct _E_Config
    int                       menu_eap_comment_show; // GUI
    int                       menu_favorites_show; // GUI
    int                       menu_apps_show; // GUI
-   Eina_Bool                menu_icons_hide; // GUI
+   Eina_Bool                 menu_icons_hide; // GUI
    int                       menu_gadcon_client_toplevel; // GUI
    int                       fullscreen_policy; // GUI
    const char               *exebuf_term_cmd; // GUI
@@ -207,10 +201,6 @@ struct _E_Config
    int                       desklock_on_suspend; // GUI
    int                       desklock_autolock_screensaver; // GUI
    double                    desklock_post_screensaver_time; // GUI
-   int                       desklock_autolock_idle; // GUI
-   double                    desklock_autolock_idle_timeout; // GUI
-   int                       desklock_use_custom_desklock; // GUI
-   const char               *desklock_custom_desklock_cmd; // GUI
    unsigned char             desklock_ask_presentation; // GUI
    double                    desklock_ask_presentation_timeout; // GUI
 
@@ -253,10 +243,21 @@ struct _E_Config
    int                       clientlist_max_caption_len; // GUI
 
    int                       mouse_hand; //GUI
-   int                       mouse_accel_numerator; // GUI
-   int                       mouse_accel_denominator; // GUI
+   double                    mouse_accel; // GUI
    int                       mouse_accel_threshold; // GUI
+   unsigned char             mouse_natural_scroll; // GUI
+   unsigned char             mouse_emulate_middle_button; // GUI
+
+   double                    touch_accel; // GUI
+   unsigned char             touch_natural_scroll; // GUI
+   unsigned char             touch_emulate_middle_button; // GUI
    unsigned char             touch_tap_to_click; // GUI
+   unsigned char             touch_clickpad; // GUI
+   unsigned char             touch_scrolling_2finger; // GUI
+   unsigned char             touch_scrolling_edge; // GUI
+   unsigned char             touch_scrolling_circular; // GUI
+   unsigned char             touch_scrolling_horiz; // GUI
+   unsigned char             touch_palm_detect; // GUI
 
    int                       border_raise_on_mouse_action; // GUI
    int                       border_raise_on_focus; // GUI
@@ -290,6 +291,8 @@ struct _E_Config
 
    int                       thumb_nice;
 
+   unsigned char             gesture_open_input_devices; // GUI
+
    int                       ping_clients_interval; // GUI
 
    int                       thumbscroll_enable; // GUI
@@ -326,8 +329,10 @@ struct _E_Config
       double        max; // GUI
       double        factor; // GUI
       int           base_dpi; // GUI
+      int           xapp_base_dpi; // GUI
       unsigned char use_dpi; // GUI
       unsigned char use_custom; // GUI
+      unsigned char set_xapp_dpi; // GUI
    } scale;
 
    unsigned char show_cursor; // GUI
@@ -348,12 +353,6 @@ struct _E_Config
       Eina_List    *actions;
    } syscon;
 
-   struct
-   {
-      unsigned char presentation; // INTERNAL
-      unsigned char offline; // INTERNAL
-   } mode;
-
    struct
    {
       double        expire_timeout;
@@ -371,9 +370,10 @@ struct _E_Config
       double        dim; // GUI
       double        transition; // GUI
       double        timer; // GUI
+      double        battery_timer; // GUI
       const char   *sysdev; // GUI
       unsigned char idle_dim; // GUI
-      E_Backlight_Mode mode; /* not saved, display-only */
+      unsigned char ddc; // GUI
    } backlight;
 
    struct
@@ -432,7 +432,7 @@ struct _E_Config
        * -zmike, 31 January 2013
        */
       const char *cur_layout; // whatever the current layout is
-      const char *selected_layout; // whatever teh current layout that the user has selected is
+      const char *selected_layout; // whatever the current layout that the user has selected is
       const char *desklock_layout;
       Eina_Bool use_cache;
    } xkb;
@@ -445,6 +445,7 @@ struct _E_Config
 
    Eina_List  *menu_applications;
    unsigned char exe_always_single_instance; // GUI
+   unsigned char explicit_date_time; // GUI
    Eina_List *screen_profiles;
 };
 
@@ -457,6 +458,7 @@ struct E_Config_Bindings
    Eina_List  *signal_bindings; // GUI
    Eina_List  *wheel_bindings; // GUI
    Eina_List  *acpi_bindings; // GUI
+   Eina_List  *swipe_bindings;
 };
 
 struct _E_Config_Desklock_Background
@@ -485,8 +487,6 @@ struct _E_Config_Module
 {
    const char   *name;
    unsigned char enabled;
-   unsigned char delayed;
-   int           priority;
 };
 
 struct _E_Config_Binding_Mouse
@@ -549,6 +549,14 @@ struct _E_Config_Binding_Acpi
    const char *action, *params;
 };
 
+struct _E_Config_Binding_Swipe
+{
+   int context;
+   unsigned int fingers;
+   double direction, length, error;
+   const char *action, *params;
+};
+
 struct _E_Config_Desktop_Background
 {
    int         zone;
@@ -682,6 +690,7 @@ E_API void e_config_binding_mouse_free(E_Config_Binding_Mouse *ebm);
 E_API void e_config_binding_edge_free(E_Config_Binding_Edge *ebe);
 E_API void e_config_binding_key_free(E_Config_Binding_Key *ebk);
 E_API void e_config_binding_acpi_free(E_Config_Binding_Acpi *eba);
+E_API void e_config_binding_swipe_free(E_Config_Binding_Swipe *eba);
 
 extern E_API E_Config *e_config;
 extern E_API E_Config_Bindings *e_bindings;
