@@ -263,6 +263,7 @@ e_menu_activate_key(E_Menu *m, E_Zone *zone, int x, int y, int w, int h, int dir
         m->cur.y = y;
         if ((m->cur.y + m->cur.h) > (m->zone->y + m->zone->h))
           m->cur.y = y + h - m->cur.h;
+        e_menu_idler_before();
         _e_menu_activate_first();
         break;
 
@@ -270,6 +271,7 @@ e_menu_activate_key(E_Menu *m, E_Zone *zone, int x, int y, int w, int h, int dir
         _e_menu_realize(m);
         m->cur.x = x + w;
         m->cur.y = y;
+        e_menu_idler_before();
         _e_menu_activate_first();
         break;
 
@@ -279,6 +281,7 @@ e_menu_activate_key(E_Menu *m, E_Zone *zone, int x, int y, int w, int h, int dir
         if ((m->cur.x + m->cur.w) > (m->zone->x + m->zone->w))
           m->cur.x = x + w - m->cur.w;
         m->cur.y = y - m->cur.h;
+        e_menu_idler_before();
         _e_menu_activate_last();
         break;
 
@@ -290,24 +293,25 @@ e_menu_activate_key(E_Menu *m, E_Zone *zone, int x, int y, int w, int h, int dir
         m->cur.y = y + h;
         if ((m->cur.y + m->cur.h) > (m->zone->y + m->zone->h))
           m->cur.y = y + h - m->cur.h;
+        e_menu_idler_before();
         _e_menu_activate_first();
         break;
 
       case E_MENU_POP_DIRECTION_AUTO:
-      {
-         int pos = 0;
+          {
+             int pos = 0;
 
-         pos = _e_menu_auto_place(m, x, y, w, h);
-         if (pos == 4)
-           _e_menu_activate_last();
-         else
-           _e_menu_activate_first();
-      }
+             pos = _e_menu_auto_place(m, x, y, w, h);
+             e_menu_idler_before();
+             if (pos == 4) _e_menu_activate_last();
+             else _e_menu_activate_first();
+          }
       break;
 
       default:
         m->cur.x = x + w;
         m->cur.y = y + h;
+        e_menu_idler_before();
         _e_menu_activate_first();
         break;
      }
@@ -343,12 +347,14 @@ e_menu_activate_mouse(E_Menu *m, E_Zone *zone, int x, int y, int w, int h, int d
         m->cur.y = y;
         if ((m->cur.y + m->cur.h) > (m->zone->y + m->zone->h))
           m->cur.y = y + h - m->cur.h;
+        e_menu_idler_before();
         break;
 
       case E_MENU_POP_DIRECTION_RIGHT:
         _e_menu_realize(m);
         m->cur.x = x + w;
         m->cur.y = y;
+        e_menu_idler_before();
         break;
 
       case E_MENU_POP_DIRECTION_UP:
@@ -357,6 +363,7 @@ e_menu_activate_mouse(E_Menu *m, E_Zone *zone, int x, int y, int w, int h, int d
         if ((m->cur.x + m->cur.w) > (m->zone->x + m->zone->w))
           m->cur.x = x + w - m->cur.w;
         m->cur.y = y - m->cur.h;
+        e_menu_idler_before();
         break;
 
       case E_MENU_POP_DIRECTION_DOWN:
@@ -367,15 +374,18 @@ e_menu_activate_mouse(E_Menu *m, E_Zone *zone, int x, int y, int w, int h, int d
         m->cur.y = y + h;
         if ((m->cur.y + m->cur.h) > (m->zone->y + m->zone->h))
           m->cur.y = y + h - m->cur.h;
+        e_menu_idler_before();
         break;
 
       case E_MENU_POP_DIRECTION_AUTO:
+        e_menu_idler_before();
         _e_menu_auto_place(m, x, y, w, h);
         break;
 
       default:
         m->cur.x = x + w;
         m->cur.y = y + h;
+        e_menu_idler_before();
         break;
      }
    pmi = _e_menu_item_active_get();
@@ -413,6 +423,7 @@ e_menu_activate(E_Menu *m, E_Zone *zone, int x, int y, int w, int h, int dir)
         m->cur.y = y;
         if ((m->cur.y + m->cur.h) > (m->zone->y + m->zone->h))
           m->cur.y = y + h - m->cur.h;
+        e_menu_idler_before();
         _e_menu_activate_first();
         break;
 
@@ -420,6 +431,7 @@ e_menu_activate(E_Menu *m, E_Zone *zone, int x, int y, int w, int h, int dir)
         _e_menu_realize(m);
         m->cur.x = x + w;
         m->cur.y = y;
+        e_menu_idler_before();
         _e_menu_activate_first();
         break;
 
@@ -429,6 +441,7 @@ e_menu_activate(E_Menu *m, E_Zone *zone, int x, int y, int w, int h, int dir)
         if ((m->cur.x + m->cur.w) > (m->zone->x + m->zone->w))
           m->cur.x = x + w - m->cur.w;
         m->cur.y = y - m->cur.h;
+        e_menu_idler_before();
         _e_menu_activate_last();
         break;
 
@@ -440,22 +453,53 @@ e_menu_activate(E_Menu *m, E_Zone *zone, int x, int y, int w, int h, int dir)
         m->cur.y = y + h;
         if ((m->cur.y + m->cur.h) > (m->zone->y + m->zone->h))
           m->cur.y = y + h - m->cur.h;
+        e_menu_idler_before();
         _e_menu_activate_first();
         break;
 
       case E_MENU_POP_DIRECTION_AUTO:
+        e_menu_idler_before();
         _e_menu_auto_place(m, x, y, w, h);
         break;
 
       default:
         m->cur.x = x + w;
         m->cur.y = y + h;
+        e_menu_idler_before();
         break;
      }
    pmi = _e_menu_item_active_get();
    if (pmi) e_menu_item_active_set(pmi, 0);
 }
 
+static void
+_e_menu_item_unhilight(E_Menu_Item *mi)
+{
+   if (!mi->hilighted) return;
+   mi->hilighted = 0;
+   if (mi->bg_object)
+     edje_object_signal_emit(mi->bg_object, "e,state,unselected", "e");
+   if (mi->icon_bg_object)
+     edje_object_signal_emit(mi->icon_bg_object, "e,state,unselected", "e");
+   if (isedje(mi->label_object))
+     edje_object_signal_emit(mi->label_object, "e,state,unselected", "e");
+   if (isedje(mi->submenu_object))
+     edje_object_signal_emit(mi->submenu_object, "e,state,unselected", "e");
+   if (isedje(mi->toggle_object))
+     edje_object_signal_emit(mi->toggle_object, "e,state,unselected", "e");
+   if (mi->icon_key)
+     {
+        if (mi->icon_object)
+          {
+             if (isedje(mi->icon_object))
+               edje_object_signal_emit(mi->icon_object, "e,state,unselected", "e");
+             else
+               e_icon_selected_set(mi->icon_object, EINA_FALSE);
+          }
+     }
+   edje_object_signal_emit(mi->menu->bg_object, "e,state,unselected", "e");
+}
+
 E_API void
 e_menu_deactivate(E_Menu *m)
 {
@@ -475,6 +519,8 @@ e_menu_deactivate(E_Menu *m)
    m->active = 0;
    if (m->post_deactivate_cb.func)
      m->post_deactivate_cb.func(m->post_deactivate_cb.data, m);
+   if (m->parent_item)
+     _e_menu_item_unhilight(m->parent_item);
 }
 
 E_API int
@@ -1030,6 +1076,7 @@ e_menu_item_active_set(E_Menu_Item *mi, int active)
                _e_menu_submenu_deactivate(_e_prev_active_menu_item);
           }
         mi->active = 1;
+        mi->hilighted = 1;
         _e_active_menu_item = mi;
         if (mi->bg_object)
           edje_object_signal_emit(mi->bg_object, "e,state,selected", "e");
@@ -1060,27 +1107,8 @@ e_menu_item_active_set(E_Menu_Item *mi, int active)
         mi->active = 0;
         _e_prev_active_menu_item = mi;
         _e_active_menu_item = NULL;
-        if (mi->bg_object)
-          edje_object_signal_emit(mi->bg_object, "e,state,unselected", "e");
-        if (mi->icon_bg_object)
-          edje_object_signal_emit(mi->icon_bg_object, "e,state,unselected", "e");
-        if (isedje(mi->label_object))
-          edje_object_signal_emit(mi->label_object, "e,state,unselected", "e");
-        if (isedje(mi->submenu_object))
-          edje_object_signal_emit(mi->submenu_object, "e,state,unselected", "e");
-        if (isedje(mi->toggle_object))
-          edje_object_signal_emit(mi->toggle_object, "e,state,unselected", "e");
-        if (mi->icon_key)
-          {
-             if (mi->icon_object)
-               {
-                  if (isedje(mi->icon_object))
-                    edje_object_signal_emit(mi->icon_object, "e,state,unselected", "e");
-                  else
-                    e_icon_selected_set(mi->icon_object, EINA_FALSE);
-               }
-          }
-        edje_object_signal_emit(mi->menu->bg_object, "e,state,unselected", "e");
+        if (!((mi->submenu) && (mi->submenu->active)))
+          _e_menu_item_unhilight(mi);
      }
    _e_menu_list_free_unref(tmp);
 }
@@ -2355,6 +2383,17 @@ _e_menu_activate_previous(void)
      {
         if (mi->menu->parent_item)
           {
+             Eina_List *l;
+             E_Menu_Item *mi2;
+
+             EINA_LIST_FOREACH(mi->menu->items, l, mi2)
+               {
+                  if (!mi2->disable)
+                    {
+                       e_menu_item_active_set(mi2, 0);
+                       _e_menu_item_unhilight(mi2);
+                    }
+               }
              mi = mi->menu->parent_item;
              e_menu_item_active_set(mi, 1);
              _e_menu_item_ensure_onscreen(mi);
