@@ -16,10 +16,13 @@ static Eina_Bool init = EINA_FALSE;
 static inline void
 _e_pointer_theme_buf(E_Pointer *ptr, char cursor[1024])
 {
+   const char *type = ptr->type;
+
+   if (!type) type = "default";
    if (ptr->color)
-     snprintf(cursor, 1024, "e/pointer/enlightenment/%s/color", ptr->type);
+     snprintf(cursor, 1024, "e/pointer/enlightenment/%s/color", type);
    else
-     snprintf(cursor, 1024, "e/pointer/enlightenment/%s/mono", ptr->type);
+     snprintf(cursor, 1024, "e/pointer/enlightenment/%s/mono", type);
 }
 
 static inline void
@@ -922,7 +925,9 @@ e_pointer_reset(E_Pointer *ptr)
    E_FREE_LIST(ptr->stack, _e_pointer_stack_free);
 
    eina_stringshare_del(ptr->type);
+   ptr->type = NULL;
    eina_stringshare_del(ptr->deferred_type);
+   ptr->deferred_type = NULL;
 
    /* reset pointer to default */
    e_pointer_type_push(ptr, ptr, "default");
