@@ -26,8 +26,6 @@
 #ifdef HAVE_ALLOCA_H
 # include <alloca.h>
 #endif
-#include <sys/types.h>
-#include <pwd.h>
 #include <signal.h>
 #ifdef HAVE_PRCTL
 # include <sys/prctl.h>
@@ -200,7 +198,7 @@ e_setuid_setup(uid_t *caller_uid, gid_t *caller_gid, char **caller_user, char **
 # ifdef HAVE_ENVIRON
    if (environ)
      {
-        int again;
+        int i, again;
         // go over environment array again and again... safely
         do
           {
