@@ -119,6 +119,7 @@ const E_Intl_Pair basic_language_predefined_pairs[] = {
    {"fo_FO.UTF-8", "fo_flag.png", "Føroyskt"},
    {"fr_CH.UTF-8", "ch_flag.png", "Français (Suisse)"},
    {"fr_FR.UTF-8", "fr_flag.png", "Français"},
+   {"gl_ES.UTF-8", "es_flag.png", "Galego"},
    {"he_HE.UTF-8", "il_flag.png", "עברית"},
    {"hr_HR.UTF-8", "hr_flag.png", "Hrvatski"},
    {"hu_HU.UTF-8", "hu_flag.png", "Magyar"},
@@ -133,12 +134,14 @@ const E_Intl_Pair basic_language_predefined_pairs[] = {
    {"nl_NL.UTF-8", "nl_flag.png", "Nederlands"},
    {"pl_PL.UTF-8", "pl_flag.png", "Polski"},
    {"pt_BR.UTF-8", "br_flag.png", "Português"},
+   {"ro_RO.UTF-8", "ro_flag.png", "Română"},
    {"ru_RU.UTF-8", "ru_flag.png", "Русский"},
    {"sk_SK.UTF-8", "sk_flag.png", "Slovenčina"},
    {"sl_SI.UTF-8", "si_flag.png", "Slovenščina"},
    {"sv_SE.UTF-8", "se_flag.png", "Svenska"},
    {"tr_TR.UTF-8", "tr_flag.png", "Türkçe"},
    {"uk_UK.UTF-8", "ua_flag.png", "Українська мова"},
+   {"vi_VN UTF-8", "vn_flag.png", "Tiếng Việt"},
    {"zh_CN.UTF-8", "cn_flag.png", "中文 (简体)"},
    {"zh_TW.UTF-8", "tw_flag.png", "中文 (繁體)"},
    { NULL, NULL, NULL }
