@@ -19,6 +19,7 @@ struct _E_Config_Dialog_Data
    Evas_Object *modes_obj;
    Evas_Object *rotations_obj;
    Evas_Object *enabled_obj;
+   Evas_Object *ignore_disconnect_obj;
    Evas_Object *priority_obj;
    Evas_Object *rel_mode_obj;
    Evas_Object *rel_to_obj;
@@ -32,6 +33,7 @@ struct _E_Config_Dialog_Data
    int hotplug;
    int acpi;
    int screen;
+   double hotplug_response;
    E_Randr2_Policy policy;
 };
 
@@ -104,6 +106,7 @@ _create_data(E_Config_Dialog *cfd EINA_UNUSED)
    cfdata->hotplug = !e_randr2_cfg->ignore_hotplug_events;
    cfdata->acpi = !e_randr2_cfg->ignore_acpi_events;
    cfdata->policy = e_randr2_cfg->default_policy;
+   cfdata->hotplug_response = e_randr2_cfg->hotplug_response;
    return cfdata;
 }
 
@@ -117,6 +120,7 @@ _free_data(E_Config_Dialog *cfd EINA_UNUSED, E_Config_Dialog_Data *cfdata)
      {
         eina_stringshare_del(cs->id);
         eina_stringshare_del(cs->rel_to);
+        eina_stringshare_del(cs->custom_label_screen);
         eina_stringshare_del(cs->profile);
         free(cs);
      }
@@ -127,6 +131,14 @@ _free_data(E_Config_Dialog *cfd EINA_UNUSED, E_Config_Dialog_Data *cfdata)
    E_FREE(cfdata);
 }
 
+static void
+_cb_hotplug_response_changed(void *data EINA_UNUSED, Evas_Object *obj, void *event EINA_UNUSED)
+{
+   E_Config_Dialog_Data *cfdata = data;
+   cfdata->hotplug_response = elm_slider_value_get(obj);
+   e_config_dialog_changed_set(cfdata->cfd, EINA_TRUE);
+}
+
 static void
 _cb_restore_changed(void *data EINA_UNUSED, Evas_Object *obj, void *event EINA_UNUSED)
 {
@@ -353,11 +365,15 @@ _basic_screen_info_fill(E_Config_Dialog_Data *cfdata, E_Config_Randr2_Screen *cs
    Elm_Object_Item *it, *it_sel;
    Elm_Widget_Item *iwt;
    void *dt;
+   const char *info_screen = "";
 
    if (!s) return;
    // fill all the screen status info
+   info_screen = s->info.screen;
+   if ((cs->custom_label_screen) && (cs->custom_label_screen[0])) info_screen = cs->custom_label_screen;
+   printf("RRR: [%s] [%s]", s->info.name, info_screen);
    elm_object_text_set(cfdata->name_obj, s->info.name);
-   elm_object_text_set(cfdata->screen_obj, s->info.screen);
+   elm_object_text_set(cfdata->screen_obj, info_screen);
    elm_check_state_set(cfdata->lid_obj, s->info.is_lid);
    elm_check_state_set(cfdata->backlight_obj, s->info.backlight);
    snprintf(buf, sizeof(buf), "%imm x %imm", s->info.size.w, s->info.size.h);
@@ -376,7 +392,10 @@ _basic_screen_info_fill(E_Config_Dialog_Data *cfdata, E_Config_Randr2_Screen *cs
           {
              mode_cbdata->cfdata = cfdata;
              mode_cbdata->mode = *m;
-             snprintf(buf, sizeof(buf), "%ix%i @ %1.2fHz", m->w, m->h, m->refresh);
+             if (m->flags & ECORE_X_RANDR_MODE_DOUBLE_SCAN)
+               snprintf(buf, sizeof(buf), "%ix%i @ %1.2fHz (Doublescan)", m->w, m->h, m->refresh / 2);
+             else
+               snprintf(buf, sizeof(buf), "%ix%i @ %1.2fHz", m->w, m->h, m->refresh);
              it = elm_list_item_append(cfdata->modes_obj, buf, NULL, NULL, _cb_mode_set, mode_cbdata);
              cfdata->freelist = eina_list_append(cfdata->freelist, mode_cbdata);
              /* printf("mode add %p %p %p\n", mode_cbdata, cfdata->modes_obj, it); */
@@ -442,6 +461,7 @@ _basic_screen_info_fill(E_Config_Dialog_Data *cfdata, E_Config_Randr2_Screen *cs
    elm_list_go(cfdata->rotations_obj);
 
    elm_check_state_set(cfdata->enabled_obj, cs->enabled);
+   elm_check_state_set(cfdata->ignore_disconnect_obj, cs->ignore_disconnect);
 
    elm_slider_value_set(cfdata->priority_obj, cs->priority);
 
@@ -745,6 +765,29 @@ _cb_enabled_changed(void *data, Evas_Object *obj, void *event EINA_UNUSED)
    e_config_dialog_changed_set(cfdata->cfd, EINA_TRUE);
 }
 
+static void
+_cb_label_screen_edited(void *data, Evas_Object *obj, void *event EINA_UNUSED)
+{
+   E_Config_Dialog_Data *cfdata = data;
+   E_Config_Randr2_Screen *cs = _config_screen_find(cfdata);
+   const char *str = elm_object_text_get(obj);
+   if (!cs) return;
+   printf("RR: change screen [%s]\n", str);
+   eina_stringshare_replace(&(cs->custom_label_screen), str);
+   e_config_dialog_changed_set(cfdata->cfd, EINA_TRUE);
+}
+
+static void
+_cb_ignore_disconnect_changed(void *data, Evas_Object *obj, void *event EINA_UNUSED)
+{
+   E_Config_Dialog_Data *cfdata = data;
+   E_Config_Randr2_Screen *cs = _config_screen_find(cfdata);
+   if (!cs) return;
+   cs->ignore_disconnect = elm_check_state_get(obj);
+   printf("RR: ignore_disconnect = %i\n", cs->ignore_disconnect);
+   e_config_dialog_changed_set(cfdata->cfd, EINA_TRUE);
+}
+
 static void
 _policy_text_update(E_Config_Dialog_Data *cfdata)
 {
@@ -811,8 +854,9 @@ _basic_create(E_Config_Dialog *cfd, Evas *evas EINA_UNUSED, E_Config_Dialog_Data
    Evas_Object *win = cfd->dia->win;
    Evas_Object *o, *bx, *tb, *bx2;
    Eina_List *l;
-   E_Randr2_Screen *s, *first = NULL;
+   E_Randr2_Screen *s, *sc = NULL, *first = NULL;
    E_Config_Randr2_Screen *first_cfg = NULL;
+   E_Zone *zone;
    int i;
 
    e_dialog_resizable_set(cfd->dia, 1);
@@ -838,13 +882,18 @@ _basic_create(E_Config_Dialog *cfd, Evas *evas EINA_UNUSED, E_Config_Dialog_Data
    cfdata->screens = NULL;
    cfdata->screen_items = NULL;
    i = 0;
+
+   zone = e_zone_current_get();
+   if (zone)
+     sc = e_randr2_screen_id_find(zone->randr2_id);
+
    EINA_LIST_FOREACH(e_randr2->screens, l, s)
      {
         Elm_Object_Item *it = NULL;
 
         if (s->info.connected)
           {
-             E_Config_Randr2_Screen *cs;
+             E_Config_Randr2_Screen *cs, *cs2;
 
              cs = calloc(1, sizeof(E_Config_Randr2_Screen));
              if (cs)
@@ -861,8 +910,15 @@ _basic_create(E_Config_Dialog *cfd, Evas *evas EINA_UNUSED, E_Config_Dialog_Data
                   cs->priority = s->config.priority;
                   cs->rel_mode = s->config.relative.mode;
                   cs->enabled = s->config.enabled;
+                  cs->ignore_disconnect = s->config.ignore_disconnect;
                   if (s->config.profile)
                     cs->profile = eina_stringshare_add(s->config.profile);
+                  if (e_randr2_cfg)
+                    {
+                       cs2 = e_randr2_config_screen_find(s, e_randr2_cfg);
+                       if ((cs2) && (cs2->custom_label_screen) && (cs2->custom_label_screen[0]))
+                         cs->custom_label_screen = eina_stringshare_add(cs2->custom_label_screen);
+                    }
                   cs->scale_multiplier = s->config.scale_multiplier;
                   cfdata->screens = eina_list_append(cfdata->screens, cs);
                   it = elm_hoversel_item_add(o, s->info.name,
@@ -880,6 +936,16 @@ _basic_create(E_Config_Dialog *cfd, Evas *evas EINA_UNUSED, E_Config_Dialog_Data
                             elm_object_text_set(o, s->info.name);
                           }
                     }
+                  else if (!first && sc)
+                    {
+                       if (s->id == sc->id)
+                         {
+                            first = s;
+                            first_cfg = cs;
+                            cfdata->screen = i;
+                            elm_object_text_set(o, s->info.name);
+                         }
+                    }
                   else
                     {
                        if (!first)
@@ -913,6 +979,7 @@ _basic_create(E_Config_Dialog *cfd, Evas *evas EINA_UNUSED, E_Config_Dialog_Data
    evas_object_size_hint_weight_set(o, EVAS_HINT_EXPAND, 0.0);
    evas_object_size_hint_align_set(o, EVAS_HINT_FILL, 0.5);
    elm_table_pack(tb, o, 0, 2, 1, 1);
+   evas_object_smart_callback_add(o, "changed,user", _cb_label_screen_edited, cfdata);
    evas_object_show(o);
    cfdata->screen_obj = o;
 
@@ -940,11 +1007,18 @@ _basic_create(E_Config_Dialog *cfd, Evas *evas EINA_UNUSED, E_Config_Dialog_Data
    cfdata->size_obj = o;
 
 ///////////////////////////////////////////////////////////////////////
+
+   bx2 = elm_box_add(win);
+   evas_object_size_hint_weight_set(bx2, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
+   evas_object_size_hint_align_set(bx2, EVAS_HINT_FILL, EVAS_HINT_FILL);
+   elm_table_pack(tb, bx2, 1, 0, 1, 14);
+   evas_object_show(bx2);
+
    o = elm_list_add(win);
    elm_scroller_content_min_limit(o, 1, 0);
    evas_object_size_hint_weight_set(o, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(o, EVAS_HINT_FILL, EVAS_HINT_FILL);
-   elm_table_pack(tb, o, 1, 0, 1, 14);
+   elm_box_pack_end(bx2, o);
    evas_object_show(o);
    cfdata->modes_obj = o;
 
@@ -965,6 +1039,15 @@ _basic_create(E_Config_Dialog *cfd, Evas *evas EINA_UNUSED, E_Config_Dialog_Data
    evas_object_smart_callback_add(o, "changed", _cb_enabled_changed, cfdata);
    cfdata->enabled_obj = o;
 
+   o = elm_check_add(win);
+   evas_object_size_hint_weight_set(o, 0.0, 0.0);
+   evas_object_size_hint_align_set(o, EVAS_HINT_FILL, 0.5);
+   elm_object_text_set(o, _("Ignore Disconnect"));
+   elm_table_pack(tb, o, 2, 5, 1, 1);
+   evas_object_show(o);
+   evas_object_smart_callback_add(o, "changed", _cb_ignore_disconnect_changed, cfdata);
+   cfdata->ignore_disconnect_obj = o;
+
    o = elm_slider_add(win);
    evas_object_size_hint_weight_set(o, 0.0, 0.0);
    evas_object_size_hint_align_set(o, EVAS_HINT_FILL, 0.5);
@@ -972,7 +1055,7 @@ _basic_create(E_Config_Dialog *cfd, Evas *evas EINA_UNUSED, E_Config_Dialog_Data
    elm_slider_unit_format_set(o, "%3.0f");
    elm_slider_span_size_set(o, 100);
    elm_slider_min_max_set(o, 0, 100);
-   elm_table_pack(tb, o, 2, 5, 1, 1);
+   elm_table_pack(tb, o, 2, 6, 1, 1);
    evas_object_show(o);
    evas_object_smart_callback_add(o, "changed", _cb_priority_changed, cfdata);
    cfdata->priority_obj = o;
@@ -987,7 +1070,7 @@ _basic_create(E_Config_Dialog *cfd, Evas *evas EINA_UNUSED, E_Config_Dialog_Data
    elm_hoversel_item_add(o, _("Right of"), NULL, ELM_ICON_NONE, _cb_rel_mode_right_of, cfdata);
    elm_hoversel_item_add(o, _("Above"), NULL, ELM_ICON_NONE, _cb_rel_mode_above, cfdata);
    elm_hoversel_item_add(o, _("Below"), NULL, ELM_ICON_NONE, _cb_rel_mode_below, cfdata);
-   elm_table_pack(tb, o, 2, 6, 1, 1);
+   elm_table_pack(tb, o, 2, 7, 1, 1);
    evas_object_show(o);
    cfdata->rel_mode_obj = o;
 
@@ -1007,7 +1090,7 @@ _basic_create(E_Config_Dialog *cfd, Evas *evas EINA_UNUSED, E_Config_Dialog_Data
              cfdata->screen_items2 = eina_list_append(cfdata->screen_items2, it);
           }
      }
-   elm_table_pack(tb, o, 2, 7, 1, 1);
+   elm_table_pack(tb, o, 2, 8, 1, 1);
    evas_object_show(o);
    cfdata->rel_to_obj = o;
 
@@ -1018,7 +1101,7 @@ _basic_create(E_Config_Dialog *cfd, Evas *evas EINA_UNUSED, E_Config_Dialog_Data
    elm_slider_unit_format_set(o, "%1.1f");
    elm_slider_span_size_set(o, 100);
    elm_slider_min_max_set(o, 0.0, 1.0);
-   elm_table_pack(tb, o, 2, 8, 1, 1);
+   elm_table_pack(tb, o, 2, 9, 1, 1);
    evas_object_show(o);
    evas_object_smart_callback_add(o, "changed", _cb_rel_align_changed, cfdata);
    cfdata->rel_align_obj = o;
@@ -1028,14 +1111,14 @@ _basic_create(E_Config_Dialog *cfd, Evas *evas EINA_UNUSED, E_Config_Dialog_Data
    elm_separator_horizontal_set(o, EINA_TRUE);
    evas_object_size_hint_weight_set(o, 0.0, 0.0);
    evas_object_size_hint_align_set(o, EVAS_HINT_FILL, 0.5);
-   elm_table_pack(tb, o, 2, 9, 1, 1);
+   elm_table_pack(tb, o, 2, 10, 1, 1);
    evas_object_show(o);
 
    o = elm_check_add(win);
    evas_object_size_hint_weight_set(o, 0.0, 0.0);
    evas_object_size_hint_align_set(o, EVAS_HINT_FILL, 0.5);
    elm_object_text_set(o, _("Use Profile"));
-   elm_table_pack(tb, o, 2, 10, 1, 1);
+   elm_table_pack(tb, o, 2, 11, 1, 1);
    evas_object_show(o);
    cfdata->use_profile_obj = o;
    evas_object_smart_callback_add(o, "changed", _cb_profile_enabled_changed, cfdata);
@@ -1043,7 +1126,7 @@ _basic_create(E_Config_Dialog *cfd, Evas *evas EINA_UNUSED, E_Config_Dialog_Data
    o = elm_list_add(win);
    evas_object_size_hint_weight_set(o, 0.0, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(o, EVAS_HINT_FILL, EVAS_HINT_FILL);
-   elm_table_pack(tb, o, 2, 11, 1, 1);
+   elm_table_pack(tb, o, 2, 12, 1, 1);
    evas_object_show(o);
    cfdata->profile_list_obj = o;
 
@@ -1051,7 +1134,7 @@ _basic_create(E_Config_Dialog *cfd, Evas *evas EINA_UNUSED, E_Config_Dialog_Data
    evas_object_size_hint_weight_set(o, 0.0, 0.0);
    evas_object_size_hint_align_set(o, EVAS_HINT_FILL, 0.5);
    elm_object_text_set(o, _("Custom Scale"));
-   elm_table_pack(tb, o, 2, 12, 1, 1);
+   elm_table_pack(tb, o, 2, 13, 1, 1);
    evas_object_show(o);
    cfdata->scale_custom_obj = o;
    evas_object_smart_callback_add(o, "changed", _cb_custom_scale_changed, cfdata);
@@ -1064,7 +1147,7 @@ _basic_create(E_Config_Dialog *cfd, Evas *evas EINA_UNUSED, E_Config_Dialog_Data
    elm_slider_span_size_set(o, 100);
    elm_slider_min_max_set(o, 0.5, 5.5);
    elm_slider_value_set(o, elm_config_scale_get());
-   elm_table_pack(tb, o, 2, 13, 1, 1);
+   elm_table_pack(tb, o, 2, 14, 1, 1);
    evas_object_show(o);
    cfdata->scale_value_obj = o;
    evas_object_smart_callback_add(o, "changed", _cb_scale_value_changed, cfdata);
@@ -1081,6 +1164,18 @@ _basic_create(E_Config_Dialog *cfd, Evas *evas EINA_UNUSED, E_Config_Dialog_Data
    evas_object_show(o);
    bx2 = o;
 
+   o = elm_slider_add(win);
+   evas_object_size_hint_weight_set(o, EVAS_HINT_EXPAND, 0.0);
+   evas_object_size_hint_align_set(o, EVAS_HINT_FILL, EVAS_HINT_FILL);
+   elm_object_text_set(o, _("Response"));
+   elm_slider_unit_format_set(o, "%1.1f sec");
+   elm_slider_span_size_set(o, 88);
+   elm_slider_min_max_set(o, 0.2, 9.0);
+   elm_slider_value_set(o, cfdata->hotplug_response);
+   elm_box_pack_end(bx2, o);
+   evas_object_show(o);
+   evas_object_smart_callback_add(o, "changed", _cb_hotplug_response_changed, cfdata);
+
    o = elm_check_add(win);
    evas_object_size_hint_weight_set(o, EVAS_HINT_EXPAND, 0.0);
    evas_object_size_hint_align_set(o, EVAS_HINT_FILL, EVAS_HINT_FILL);
@@ -1137,6 +1232,7 @@ _basic_apply(E_Config_Dialog *cfd EINA_UNUSED, E_Config_Dialog_Data *cfdata)
    e_randr2_cfg->ignore_hotplug_events = !cfdata->hotplug;
    e_randr2_cfg->ignore_acpi_events = !cfdata->acpi;
    e_randr2_cfg->default_policy = cfdata->policy;
+   e_randr2_cfg->hotplug_response = cfdata->hotplug_response;
 
    printf("APPLY....................\n");
    EINA_LIST_FOREACH(cfdata->screens, l, cs2)
@@ -1161,12 +1257,16 @@ _basic_apply(E_Config_Dialog *cfd EINA_UNUSED, E_Config_Dialog_Data *cfdata)
         cs->rotation = cs2->rotation;
         cs->priority = cs2->priority;
         cs->rel_mode = cs2->rel_mode;
+        if (cs->custom_label_screen) eina_stringshare_del(cs->custom_label_screen);
+        cs->custom_label_screen = NULL;
+        if (cs2->custom_label_screen) cs->custom_label_screen = eina_stringshare_add(cs2->custom_label_screen);
         if (cs->profile) eina_stringshare_del(cs->profile);
         cs->profile = NULL;
         if (cs2->profile) cs->profile = eina_stringshare_add(cs2->profile);
         cs->scale_multiplier = cs2->scale_multiplier;
         printf("APPLY %s .... rel mode %i\n", cs->id, cs->rel_mode);
         cs->enabled = cs2->enabled;
+        cs->ignore_disconnect = cs2->ignore_disconnect;
      }
    e_randr2_config_save();
    e_randr2_config_apply();
