@@ -14,9 +14,9 @@ struct _E_Config_Dialog_Data
    Eina_List       *widget_themes;
    const char      *widget_theme;
    int              enable_xsettings;
+   Eina_List       *icon_themes;
    int              match_e17_theme;
    int              match_e17_icon_theme;
-   Eina_List       *icon_themes;
    const char      *icon_theme;
    int              icon_overrides;
    //int              enable_icon_theme; // We just need to check whether override or match icon theme is set.
@@ -159,13 +159,21 @@ _basic_apply(E_Config_Dialog *cfd, E_Config_Dialog_Data *cfdata)
    e_config->xsettings.match_e17_icon_theme = cfdata->match_e17_icon_theme;
    e_config_save_queue();
 
-   if (cfdata->match_e17_icon_theme &&
-       strcmp(e_config->icon_theme, elm_config_icon_theme_get()))
+/*   if (//cfdata->match_e17_icon_theme &&
+       !!strcmp(e_config->icon_theme, elm_config_icon_theme_get()))
      {
         elm_config_icon_theme_set(e_config->icon_theme);
         elm_config_all_flush();
         elm_config_save();
      }
+   else */
+   if (//cfdata->match_e17_icon_theme &&
+       !!strcmp("_Elementary_Icon_Theme", elm_config_icon_theme_get()))
+     {
+        elm_config_icon_theme_set("_Elementary_Icon_Theme");
+        elm_config_all_flush();
+        elm_config_save();
+     }
    e_util_env_set("E_ICON_THEME", e_config->icon_theme);
 
    ev = E_NEW(E_Event_Config_Icon_Theme, 1);
@@ -487,12 +495,13 @@ _basic_create(E_Config_Dialog *cfd EINA_UNUSED, Evas *evas, E_Config_Dialog_Data
    oc = e_widget_check_add(evas, _("Enable X Application Settings"),
                            &(cfdata->enable_xsettings));
    e_widget_list_object_append(ol, oc, 0, 0, 0.0);
-#endif
+
    e_widget_check_widget_disable_on_unchecked_add(oc, ilist);
    e_widget_check_widget_disable_on_unchecked_add(oc, ow);
+#endif
+
    e_widget_toolbook_page_append(otb, NULL, _("GTK Applications"), ol,
                                  1, 1, 1, 1, 0.5, 0.0);
-
    ol = e_widget_list_add(evas, 0, 0);
    ilist = e_widget_ilist_add(evas, 24, 24, &(cfdata->icon_theme));
    cfdata->gui.icon_list = ilist;
