@@ -34,6 +34,8 @@ struct _Instance
 {
    E_Gadcon_Client *gcc;
    Evas_Object     *o_cpu;
+   Ecore_Timer     *update_timer;
+   Eina_List       *strings;
 };
 
 static void      _button_cb_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info);
@@ -43,6 +45,7 @@ static Cpu_Status *_cpufreq_status_new(void);
 static void      _cpufreq_status_free(Cpu_Status *s);
 static void      _cpufreq_status_check_available(Cpu_Status *s);
 static int       _cpufreq_status_check_current(Cpu_Status *s);
+static void      _cpufreq_status_eval(Cpu_Status *status);
 static void      _cpufreq_face_update_available(Instance *inst);
 static void      _cpufreq_face_update_current(Instance *inst);
 static void      _cpufreq_face_cb_set_frequency(void *data, Evas_Object *o, const char *emission, const char *source);
@@ -114,11 +117,14 @@ static void
 _gc_shutdown(E_Gadcon_Client *gcc)
 {
    Instance *inst;
+   const char *s;
 
    inst = gcc->data;
    cpufreq_config->instances =
      eina_list_remove(cpufreq_config->instances, inst);
    evas_object_del(inst->o_cpu);
+   if (inst->update_timer) ecore_timer_del(inst->update_timer);
+   EINA_LIST_FREE(inst->strings, s) eina_stringshare_del(s);
    free(inst);
 
    if (!cpufreq_config->instances)
@@ -180,6 +186,7 @@ _button_cb_mouse_down(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_UNU
 {
    Instance *inst;
    Evas_Event_Mouse_Down *ev;
+   const char *s;
 
    inst = data;
    ev = event_info;
@@ -258,7 +265,9 @@ _button_cb_mouse_down(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_UNU
                   e_menu_item_radio_group_set(mi, 1);
                   if (!strcmp(cpufreq_config->status->cur_governor, l->data))
                     e_menu_item_toggle_set(mi, 1);
-                  e_menu_item_callback_set(mi, _cpufreq_menu_governor, l->data);
+                  s = eina_stringshare_add(l->data);
+                  inst->strings = eina_list_append(inst->strings, s);
+                  e_menu_item_callback_set(mi, _cpufreq_menu_governor, s);
                }
 
              e_menu_item_separator_set(e_menu_item_new(mo), 1);
@@ -296,7 +305,9 @@ _button_cb_mouse_down(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_UNU
                   if (cpufreq_config->powersave_governor
                       && !strcmp(cpufreq_config->powersave_governor, l->data))
                     e_menu_item_toggle_set(mi, 1);
-                  e_menu_item_callback_set(mi, _cpufreq_menu_powersave_governor, l->data);
+                  s = eina_stringshare_add(l->data);
+                  inst->strings = eina_list_append(inst->strings, s);
+                  e_menu_item_callback_set(mi, _cpufreq_menu_powersave_governor, s);
                }
 
              e_menu_item_separator_set(e_menu_item_new(mo), 1);
@@ -468,10 +479,32 @@ _button_cb_mouse_down(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_UNU
      }
 }
 
+static Eina_Bool
+_update_cb(void *data)
+{
+   Instance *inst = data;
+
+   inst->update_timer = NULL;
+   if (cpufreq_config)
+     {
+        Cpu_Status *status = _cpufreq_status_new();
+
+        if (status)
+          {
+             if (_cpufreq_status_check_current(status))
+               _cpufreq_status_eval(status);
+             else
+               _cpufreq_status_free(status);
+          }
+     }
+   return EINA_FALSE;
+}
+
 static void
 _menu_cb_post(void *data, E_Menu *m EINA_UNUSED)
 {
    Instance *inst = data;
+   const char *s;
 
    if (inst)
      e_gadcon_locked_set(inst->gcc->gadcon, 0);
@@ -495,6 +528,9 @@ _menu_cb_post(void *data, E_Menu *m EINA_UNUSED)
    if (cpufreq_config->menu_pstate2)
      e_object_del(E_OBJECT(cpufreq_config->menu_pstate2));
    cpufreq_config->menu_powersave = NULL;
+   EINA_LIST_FREE(inst->strings, s) eina_stringshare_del(s);
+   if (inst->update_timer) ecore_timer_del(inst->update_timer);
+   inst->update_timer = ecore_timer_add(0.5, _update_cb, inst);
 }
 
 static Eina_Bool _response = EINA_FALSE;
@@ -718,9 +754,10 @@ _cpufreq_status_check_available(Cpu_Status *s)
         s->frequencies = eina_list_sort(s->frequencies,
                                         eina_list_count(s->frequencies),
                                         _cpufreq_cb_sort);
+        /* sort is necessary because this can change and the order is reversed
+         * which breaks the theme. */
      }
 
-   /* sort is not necessary because freq_levels is already sorted */
    /* freebsd doesn't have governors */
    if (s->governors)
      {
@@ -782,7 +819,8 @@ _cpufreq_status_check_available(Cpu_Status *s)
                break;
             }
           fclose(f);
-          if (strcmp(buf, "intel_pstate\n")) break;
+          if (!((!strcmp(buf, "intel_pstate\n") ||
+                 (!strcmp(buf, "intel_cpufreq\n"))))) break;
 
           if (s->frequencies)
             {
@@ -1137,19 +1175,21 @@ _cpufreq_face_cb_set_governor(void *data EINA_UNUSED, Evas_Object *obj EINA_UNUS
 }
 
 static Eina_Bool
-_cpufreq_event_cb_powersave(void *data EINA_UNUSED, int type, void *event)
+_cpufreq_event_cb_powersave(void *data EINA_UNUSED, int type EINA_UNUSED, void *event)
 {
    E_Event_Powersave_Update *ev;
    Eina_List *l;
    Eina_Bool has_powersave = EINA_FALSE;
    Eina_Bool has_conservative = EINA_FALSE;
+// don't use these at this point...
+//   Eina_Bool has_ondemand = EINA_FALSE;
+//   Eina_Bool has_interactive = EINA_FALSE;
+   const char *gov;
 
-   if (type != E_EVENT_POWERSAVE_UPDATE) return ECORE_CALLBACK_PASS_ON;
+   printf("PWSV: auto=%i\n", cpufreq_config->auto_powersave);
    if (!cpufreq_config->auto_powersave) return ECORE_CALLBACK_PASS_ON;
 
    ev = event;
-   if (!cpufreq_config->status->orig_governor)
-     cpufreq_config->status->orig_governor = eina_stringshare_add(cpufreq_config->status->cur_governor);
 
    for (l = cpufreq_config->status->governors; l; l = l->next)
      {
@@ -1157,34 +1197,53 @@ _cpufreq_event_cb_powersave(void *data EINA_UNUSED, int type, void *event)
           has_conservative = EINA_TRUE;
         else if (!strcmp(l->data, "powersave"))
           has_powersave = EINA_TRUE;
-        else if (!strcmp(l->data, "interactive"))
-          has_powersave = EINA_TRUE;
+// don't use these at this point...
+//        else if (!strcmp(l->data, "ondemand"))
+//          has_ondemand = EINA_TRUE;
+//        else if (!strcmp(l->data, "interactive"))
+//          has_interactive = EINA_TRUE;
      }
 
    switch (ev->mode)
      {
       case E_POWERSAVE_MODE_NONE:
+        printf("PWSV: none\n");
+        EINA_FALLTHROUGH;
       case E_POWERSAVE_MODE_LOW:
-        _cpufreq_set_governor(cpufreq_config->status->orig_governor);
-        eina_stringshare_del(cpufreq_config->status->orig_governor);
-        cpufreq_config->status->orig_governor = NULL;
+        printf("PWSV: low - orig=%s\n", cpufreq_config->governor);
+        gov = cpufreq_config->status->orig_governor;
+        if (!gov) gov = cpufreq_config->governor;
+        eina_stringshare_replace(&(cpufreq_config->status->orig_governor), NULL);
+        if (gov) _cpufreq_set_governor(gov);
         break;
 
       case E_POWERSAVE_MODE_MEDIUM:
+        printf("PWSV: med\n");
+        EINA_FALLTHROUGH;
       case E_POWERSAVE_MODE_HIGH:
-        if ((cpufreq_config->powersave_governor) || (has_conservative))
+        printf("PWSV: hi\n");
+        eina_stringshare_replace(&(cpufreq_config->status->orig_governor),
+                                 cpufreq_config->status->cur_governor);
+        if (cpufreq_config->powersave_governor)
           {
-             if (cpufreq_config->powersave_governor)
-               _cpufreq_set_governor(cpufreq_config->powersave_governor);
-             else
-               _cpufreq_set_governor("conservative");
+             _cpufreq_set_governor(cpufreq_config->powersave_governor);
+             break;
+          }
+        else if (has_conservative)
+          {
+             _cpufreq_set_governor("conservative");
              break;
           }
         EINA_FALLTHROUGH;
         /* no break */
 
       case E_POWERSAVE_MODE_EXTREME:
+        printf("PWSV: extreme\n");
+        EINA_FALLTHROUGH;
       case E_POWERSAVE_MODE_FREEZE:
+        printf("PWSV: freeze\n");
+        eina_stringshare_replace(&(cpufreq_config->status->orig_governor),
+                                 cpufreq_config->status->cur_governor);
         if (has_powersave)
           _cpufreq_set_governor("powersave");
         break;
@@ -1321,7 +1380,7 @@ static void
 _cpufreq_cb_frequency_check_done(void *data, Ecore_Thread *th EINA_UNUSED)
 {
    Thread_Config *thc = data;
-   e_powersave_sleeper_free(thc->sleeper);
+   if (thc->sleeper) e_powersave_sleeper_free(thc->sleeper);
    free(thc);
 }
 
@@ -1340,16 +1399,13 @@ _cpufreq_cb_frequency_check_main(void *data, Ecore_Thread *th)
         else
           _cpufreq_status_free(status);
         if (ecore_thread_check(th)) break;
-        e_powersave_sleeper_sleep(thc->sleeper, thc->interval);
+        e_powersave_sleeper_sleep(thc->sleeper, thc->interval, EINA_TRUE);
      }
 }
 
 static void
-_cpufreq_cb_frequency_check_notify(void *data EINA_UNUSED,
-                                   Ecore_Thread *th EINA_UNUSED,
-                                   void *msg)
+_cpufreq_status_eval(Cpu_Status *status)
 {
-   Cpu_Status *status = msg;
    Instance *inst;
    Eina_List *l;
    int active;
@@ -1372,7 +1428,7 @@ _cpufreq_cb_frequency_check_notify(void *data EINA_UNUSED,
         (status->cur_max_frequency != cpufreq_config->status->cur_max_frequency) ||
         (status->can_set_frequency != cpufreq_config->status->can_set_frequency)))
      freq_changed = EINA_TRUE;
-   _cpufreq_status_free(cpufreq_config->status);
+   if (cpufreq_config->status) _cpufreq_status_free(cpufreq_config->status);
    cpufreq_config->status = status;
    if (freq_changed)
      {
@@ -1401,6 +1457,14 @@ _cpufreq_cb_frequency_check_notify(void *data EINA_UNUSED,
      }
 }
 
+static void
+_cpufreq_cb_frequency_check_notify(void *data EINA_UNUSED,
+                                   Ecore_Thread *th EINA_UNUSED,
+                                   void *msg)
+{
+   _cpufreq_status_eval(msg);
+}
+
 void
 _cpufreq_poll_interval_update(void)
 {
@@ -1435,7 +1499,6 @@ E_API E_Module_Api e_modapi =
 E_API void *
 e_modapi_init(E_Module *m)
 {
-   char buf[PATH_MAX];
    Eina_List *l;
 
    conf_edd = E_CONFIG_DD_NEW("Cpufreq_Config", Config);
@@ -1497,11 +1560,10 @@ e_modapi_init(E_Module *m)
    cpufreq_config->module = m;
 
    e_gadcon_provider_register(&_gadcon_class);
-   snprintf(buf, sizeof(buf), "%s/e-module-cpufreq.edj", e_module_dir_get(m));
    e_configure_registry_category_add("advanced", 80, _("Advanced"), NULL,
                                      "preferences-advanced");
    e_configure_registry_item_add("advanced/cpufreq", 120, _("CPU Frequency"),
-                                 NULL, buf, e_int_config_cpufreq_module);
+                                 NULL, "preferences-cpu-speed", e_int_config_cpufreq_module);
    return m;
 }
 
