@@ -1517,10 +1517,8 @@ _ibar_cb_icon_mouse_in(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_UN
    E_FREE_FUNC(ic->hide_timer, ecore_timer_del);
    if (!ic->ibar->inst->ci->dont_icon_menu_mouseover)
      {
-        if (ic->show_timer)
-          ecore_timer_loop_reset(ic->show_timer);
-        else
-          ic->show_timer = ecore_timer_loop_add(0.2, _ibar_icon_mouse_in_timer, ic);
+        E_FREE_FUNC(ic->show_timer, ecore_timer_del);
+        ic->show_timer = ecore_timer_loop_add(0.2, _ibar_icon_mouse_in_timer, ic);
      }
 }
 
@@ -1950,10 +1948,21 @@ _ibar_cb_icon_move(void *data, Evas *e, Evas_Object *obj, void *event_info EINA_
         if (!zone)
           zone = eina_list_data_get(e_comp->zones);
      }
-   if (chx - (len / 2) < zone->x)
+   if ((ic->ibar->inst->orient == E_GADCON_ORIENT_LEFT) ||
+       (ic->ibar->inst->orient == E_GADCON_ORIENT_CORNER_LT) ||
+       (ic->ibar->inst->orient == E_GADCON_ORIENT_CORNER_LB))
      sig = "e,origin,left";
-   else if ((chx + (len / 2) > cw) || ((chx + (len / 2) > zone->x + zone->w)))
+   else if ((ic->ibar->inst->orient == E_GADCON_ORIENT_RIGHT) ||
+            (ic->ibar->inst->orient == E_GADCON_ORIENT_CORNER_RT) ||
+            (ic->ibar->inst->orient == E_GADCON_ORIENT_CORNER_RB))
      sig = "e,origin,right";
+   else
+     {
+        if (chx - (len / 2) < zone->x)
+          sig = "e,origin,left";
+        else if ((chx + (len / 2) > cw) || ((chx + (len / 2) > zone->x + zone->w)))
+          sig = "e,origin,right";
+     }
    _ibar_icon_signal_emit(ic, sig, "e");
 }
 
