@@ -63,18 +63,20 @@ _notification_popup_merge(E_Notification_Notify *n)
    len = strlen(popup->notif->body);
    len += strlen(n->body);
    len += 5; /* \xE2\x80\xA9 or <PS/> */
-   if (len < 8192) body_final = alloca(len + 1);
-   else body_final = malloc(len + 1);
-   /* Hack to allow e to include markup */
-   snprintf(body_final, len + 1, "%s<ps/>%s", popup->notif->body, n->body);
+   body_final = malloc(len + 1);
+   if (body_final)
+     {
+        /* Hack to allow e to include markup */
+        snprintf(body_final, len + 1, "%s<ps/>%s", popup->notif->body, n->body);
 
-   /* printf("set body %s\n", body_final); */
+        /* printf("set body %s\n", body_final); */
 
-   eina_stringshare_replace(&n->body, body_final);
+        eina_stringshare_replace(&n->body, body_final);
 
-   e_object_del(E_OBJECT(popup->notif));
-   popup->notif = n;
-   if (len >= 8192) free(body_final);
+        e_object_del(E_OBJECT(popup->notif));
+        popup->notif = n;
+        free(body_final);
+     }
 
    return popup;
 }
@@ -271,7 +273,6 @@ static Popup_Data *
 _notification_popup_new(E_Notification_Notify *n, unsigned id)
 {
    Popup_Data *popup;
-   char buf[PATH_MAX];
    Eina_List *l;
    int pos = next_pos;
    E_Zone *zone = NULL;
@@ -305,10 +306,7 @@ _notification_popup_new(E_Notification_Notify *n, unsigned id)
    popup->e = e_comp->evas;
 
    /* Setup the theme */
-   snprintf(buf, sizeof(buf), "%s/e-module-notification.edj",
-            notification_mod->dir);
    popup->theme = edje_object_add(popup->e);
-
    e_theme_edje_object_set(popup->theme,
                            "base/theme/modules/notification",
                            "e/modules/notification/main");
@@ -489,19 +487,10 @@ _notification_popup_refresh(Popup_Data *popup)
 
    if (!popup->app_icon)
      {
-        char buf[PATH_MAX];
-
-        snprintf(buf, sizeof(buf), "%s/e-module-notification.edj",
-                 notification_mod->dir);
         popup->app_icon = edje_object_add(popup->e);
-        if (!e_theme_edje_object_set(popup->app_icon,
-                                     "base/theme/modules/notification",
-                                     "e/modules/notification/logo"))
-          if (!e_theme_edje_object_set(popup->app_icon,
-                                       "base/theme/modules/notification",
-                                       "modules/notification/logo"))
-            edje_object_file_set(popup->app_icon, buf,
-                                 "modules/notification/logo");
+        e_theme_edje_object_set(popup->app_icon,
+                                "base/theme/modules/notification",
+                                "e/modules/notification/logo");
         w = width;
         h = height;
      }
