@@ -1779,7 +1779,7 @@ ui_tool_add(Evas_Object *parent, Evas_Object *tb, Evas_Object *radg, int x, int
    evas_object_show(o);
    return rad;
 }
-
+/* not used right now
 static Evas_Object *
 ui_icon_button_add(Evas_Object *parent, const char *icon)
 {
@@ -1798,6 +1798,21 @@ ui_icon_button_add(Evas_Object *parent, const char *icon)
    evas_object_show(o);
    return o;
 }
+*/
+static Evas_Object *
+ui_icon_button_standard_add(Evas_Object *parent, const char *icon)
+{
+   Evas_Object *o, *ic;
+
+   ic = o = elm_icon_add(parent);
+   elm_icon_standard_set(o, icon);
+
+   o = elm_button_add(parent);
+   elm_object_content_set(o, ic);
+   evas_object_show(ic);
+   evas_object_show(o);
+   return o;
+}
 
 Evas_Object *
 ui_edit(Evas_Object *window, Evas_Object *o_bg, E_Zone *zone,
@@ -1805,7 +1820,7 @@ ui_edit(Evas_Object *window, Evas_Object *o_bg, E_Zone *zone,
         Evas_Object **o_img_ret)
 {
    Evas *evas, *evas2;
-   Evas_Object *o, *tb, *tb2, *sc, *fr, *bx, *radg, *rad, *bx_ret;
+   Evas_Object *o, *tb, *tb2, *sc, *fr, *bx, *radg, *rad, *bx_ret, *sc2;
    int w, h;
 
    o = win = window;
@@ -1830,9 +1845,11 @@ ui_edit(Evas_Object *window, Evas_Object *o_bg, E_Zone *zone,
           }
      }
    w = (sw * zooms[zoom]) / 1000;
-   if (w < ELM_SCALE_SIZE(400)) w = 400;
+   if ((zone) && (w > (zone->w / 8))) w = zone->w / 8;
+   if (w < ELM_SCALE_SIZE(360)) w = ELM_SCALE_SIZE(360);
    h = (w * sh) / sw;
-   if ((zone) && (h > (zone->h / 2))) h = zone->h / 2;
+   if ((zone) && (h > (zone->h / 8))) h = zone->h / 8;
+   if (h < ELM_SCALE_SIZE(280)) h = ELM_SCALE_SIZE(280);
 
    if (!ec)
      {
@@ -1904,9 +1921,11 @@ ui_edit(Evas_Object *window, Evas_Object *o_bg, E_Zone *zone,
    evas_object_size_hint_align_set(o, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_min_set(o, w, h);
    evas_object_pass_events_set(o, EINA_TRUE);
-   elm_table_pack(tb, o, 0, 0, 10, 10);
+   elm_table_pack(tb, o, 0, 0, 5, 5);
 
    sc = o = elm_scroller_add(win);
+   evas_object_data_set(win, "scroll/main", o);
+   elm_object_style_set(o, "noclip");
    elm_scroller_gravity_set(o, 0.5, 0.5);
    elm_object_focus_next_object_set(o, o, ELM_FOCUS_PREVIOUS);
    elm_object_focus_next_object_set(o, o, ELM_FOCUS_NEXT);
@@ -1917,13 +1936,23 @@ ui_edit(Evas_Object *window, Evas_Object *o_bg, E_Zone *zone,
    o_scroll = sc;
    evas_object_size_hint_weight_set(o, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(o, EVAS_HINT_FILL, EVAS_HINT_FILL);
-   elm_table_pack(tb, o, 0, 0, 10, 10);
+   elm_table_pack(tb, o, 0, 0, 5, 5);
+   evas_object_show(o);
+
+   sc2 = o = elm_scroller_add(win);
+   evas_object_data_set(win, "scroll/tools", o);
+   elm_object_style_set(o, "noclip");
+   elm_scroller_content_min_limit(o, EINA_TRUE, EINA_FALSE);
+   elm_scroller_gravity_set(o, 0.0, 0.0);
+   evas_object_size_hint_weight_set(o, 0.0, EVAS_HINT_EXPAND);
+   evas_object_size_hint_align_set(o, EVAS_HINT_FILL, EVAS_HINT_FILL);
+   elm_table_pack(tb, o, 5, 0, 1, 5);
    evas_object_show(o);
 
    bx_ret = bx = o = elm_box_add(win);
    evas_object_size_hint_align_set(o, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_weight_set(o, 0.0, EVAS_HINT_EXPAND);
-   elm_table_pack(tb, o, 10, 0, 1, 10);
+   elm_object_content_set(sc2, o);
    evas_object_show(o);
 
    fr = o = elm_frame_add(win);
@@ -2069,18 +2098,21 @@ ui_edit(Evas_Object *window, Evas_Object *o_bg, E_Zone *zone,
 
    o = elm_frame_add(win);
    elm_object_style_set(o, "pad_large");
-   elm_table_pack(tb, o, 9, 0, 1, 1);
+   elm_table_pack(tb, o, 4, 0, 1, 1);
 
-   o = ui_icon_button_add(win, "zoom-in");
-   elm_table_pack(tb, o, 8, 1, 1, 1);
+   o = ui_icon_button_standard_add(win, "zoom-in");
+   elm_object_style_set(o, "overlay");
+   elm_table_pack(tb, o, 3, 1, 1, 1);
    evas_object_smart_callback_add(o, "clicked", _cb_tool_zoom_plus, NULL);
 
-   o = ui_icon_button_add(win, "zoom");
-   elm_table_pack(tb, o, 8, 2, 1, 1);
+   o = ui_icon_button_standard_add(win, "zoom-original");
+   elm_object_style_set(o, "overlay");
+   elm_table_pack(tb, o, 3, 2, 1, 1);
    evas_object_smart_callback_add(o, "clicked", _cb_tool_zoom_reset, NULL);
 
-   o = ui_icon_button_add(win, "zoom-out");
-   elm_table_pack(tb, o, 8, 3, 1, 1);
+   o = ui_icon_button_standard_add(win, "zoom-out");
+   elm_object_style_set(o, "overlay");
+   elm_table_pack(tb, o, 3, 3, 1, 1);
    evas_object_smart_callback_add(o, "clicked", _cb_tool_zoom_minus, NULL);
    evas_object_show(o);
 
