@@ -4,10 +4,11 @@
 int e_winlist_init(void);
 int e_winlist_shutdown(void);
 
-typedef enum _E_Winlist_Filter {
-  E_WINLIST_FILTER_NONE = 0,
-  E_WINLIST_FILTER_CLASS_WINDOWS = 1, /* all windows from the same class */
-  E_WINLIST_FILTER_CLASSES = 2 /* loop through classes (last selected win) */
+typedef enum _E_Winlist_Filter
+{
+   E_WINLIST_FILTER_NONE = 0,
+   E_WINLIST_FILTER_CLASS_WINDOWS = 1, /* all windows from the same class */
+   E_WINLIST_FILTER_CLASSES = 2 /* loop through classes (last selected win) */
 } E_Winlist_Filter;
 
 typedef enum E_Winlist_Activate_Type
@@ -18,6 +19,12 @@ typedef enum E_Winlist_Activate_Type
    E_WINLIST_ACTIVATE_TYPE_EDGE,
 } E_Winlist_Activate_Type;
 
+typedef enum E_Winlist_Mode
+{
+   E_WINLIST_MODE_LIST,
+   E_WINLIST_MODE_LARGE
+} E_Winlist_Mode;
+
 int  e_winlist_show(E_Zone *zone, E_Winlist_Filter filter);
 void e_winlist_hide(void);
 void e_winlist_next(void);
